package com.book.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.dao.entity.Book;
import com.book.server.dao.entity.BookCategory;
import com.book.server.dao.entity.ManageBlock;
import com.book.server.dao.entity.ManageBlockResource;
import com.book.server.dao.entity.example.BookCategoryExample;
import com.book.server.dao.entity.example.BookExample;
import com.book.server.dao.entity.example.ManageBlockExample;
import com.book.server.dao.entity.example.ManageBlockResourceExample;
import com.book.server.dao.mapper.BookCategoryMapper;
import com.book.server.dao.mapper.BookMapper;
import com.book.server.dao.mapper.ManageBlockMapper;
import com.book.server.dao.mapper.ManageBlockResourceMapper;
import com.book.server.model.VO.BlockRes;
import com.book.server.model.VO.Chapter;
import com.book.server.model.VO.BookRes;
import com.book.server.model.VO.ManageBlockResourceRes;
import com.book.server.model.VO.QueryVO;
import com.book.server.service.BookService;
import com.book.server.utils.HttpTool;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;
import java.util.*;

@Service
public class BookServiceImpl implements BookService {
    public static final Logger logger = LoggerFactory.getLogger(BookServiceImpl.class);
    @Autowired
    private BookMapper bookMapper;

    @Autowired
    private ManageBlockMapper manageBlockMapper;

    @Autowired
    private ManageBlockResourceMapper manageBlockResourceMapper;
    @Autowired
    private BookCategoryMapper bookCategoryMapper;

    public List<BookCategory> getAllCategory() {
        List<BookCategory> bookCategories = bookCategoryMapper.selectByExample(
                BookCategoryExample.newAndCreateCriteria().example()
        );
        return bookCategories;
    }


    @Override
    public PageResult<BookRes> query(QueryVO queryVO) {
        BookExample example = BookExample.newAndCreateCriteria().example();
        example.or().andNameLike("%" + queryVO.getQuery() + "%");
        example.or().andAuthorLike("%" + queryVO.getQuery() + "%");
        long count = bookMapper.countByExample(example);
        if (count <= 0) {
            return new PageResult<>(queryVO.getPage(), queryVO.getSize(), 0L, null);
        }
        example.page(queryVO.getPage(), queryVO.getSize());
        List<Book> books = bookMapper.selectByExample(example);
        //查询关联表
        List<BookCategory> allCategory = getAllCategory();
        HashMap<Integer, String> map = new HashMap<>();
        for (BookCategory bookCategory : allCategory) {
            map.put(bookCategory.getId(), bookCategory.getName());
        }
        //转存
        ArrayList<BookRes> bookResList = new ArrayList<>();
        for (Book book : books) {
            BookRes bookRes = new BookRes();
            BeanUtils.copyProperties(book,bookRes);
            bookRes.setCategoryName(map.get(book.getBookCategoryId()));
            bookResList.add(bookRes);
        }

        return new PageResult<>(queryVO.getPage(), queryVO.getSize(), count, bookResList);
    }

    @Override
    public PageResult<Book> recommendByRead(QueryVO queryVO) {
        BookExample example = BookExample.newAndCreateCriteria()
                .andSexEqualTo(queryVO.getPageId())
                .example();
        long count = bookMapper.countByExample(example);
        example.orderBy(Book.Column.readNum.desc());
        example.page(queryVO.getPage(), queryVO.getSize());
        List<Book> books = bookMapper.selectByExample(example);
        return new PageResult<>(queryVO.getPage(), queryVO.getSize(), count, books);
    }

    @Override
    public List<BlockRes> getBlockByPageId(Integer pageId) {
        ArrayList<BlockRes> list = new ArrayList<>();
        // 获取板块
        ManageBlockExample manageBlockExample = ManageBlockExample.newAndCreateCriteria()
                .andPageIdEqualTo(pageId)
                .example();
        manageBlockExample.orderBy(ManageBlock.Column.weigh.desc());
        List<ManageBlock> manageBlocks = manageBlockMapper.selectByExample(manageBlockExample);

        // 获取板块内部数据
        for (ManageBlock manageBlock : manageBlocks) {
            ManageBlockResourceExample manageBlockResourceExample = ManageBlockResourceExample.newAndCreateCriteria()
                    .andBlockIdEqualTo(manageBlock.getId())
                    .example();
            manageBlockResourceExample.orderBy(ManageBlockResource.Column.weigh.desc());
            //获取内部书籍资源
            List<ManageBlockResource> manageBlockResources = manageBlockResourceMapper.selectByExample(manageBlockResourceExample);
            ArrayList<ManageBlockResourceRes> manageBlockResourceResList = new ArrayList<>();
            if (manageBlockResources.size() > 0) {
                List<Long> collect = manageBlockResources.stream()
                        .map(ManageBlockResource::getBookId).collect(Collectors.toList());
                List<Book> books = bookMapper.selectByExample(
                        BookExample.newAndCreateCriteria()
                                .andIdIn(collect)
                                .example()
                );
                HashMap<Long, Book> bookHashMap = new HashMap<>();
                books.forEach(x->{
                    bookHashMap.put(x.getId(), x);
                });
                manageBlockResources.forEach(x->{
                    ManageBlockResourceRes manageBlockResourceRes = new ManageBlockResourceRes();
                    BeanUtils.copyProperties(x,manageBlockResourceRes);
                    manageBlockResourceResList.add(manageBlockResourceRes);

                    Book book = bookHashMap.get(x.getBookId());
                    if (book != null) {
                        manageBlockResourceRes.setBookName(book.getName());
                        manageBlockResourceRes.setBookDescription(book.getDescription());
                    }

                });
            }
            BlockRes blockRes = new BlockRes();
            blockRes.setName(manageBlock.getName());
            blockRes.setSecondName(manageBlock.getSecondName());
            blockRes.setType(manageBlock.getType());
            blockRes.setResources(manageBlockResourceResList);

            list.add(blockRes);
        }
        return list;
    }

    @Override
    public void insertBooks() {

        //插入书籍基本信息
        String bookInfoUrl = "http://new.emeixs.com/index.php/Home/Interface/listing";

        String jsonStr = HttpTool.sendGet(bookInfoUrl, "");
        JSONObject listJson = JSON.parseObject(jsonStr);
        JSONArray list = listJson.getJSONArray("list");
        Map<String, String> typeMap = new HashMap<String, String>();
        typeMap.put("15", "豪门总裁");
        typeMap.put("16", "婚恋家庭");
        typeMap.put("17", "豪门虐恋");
        typeMap.put("18", "甜宠萌宝");
        typeMap.put("19", "重生逆袭");
        typeMap.put("20", "穿越重生");
        typeMap.put("21", "权谋宫斗");
        typeMap.put("22", "经商种田");
        typeMap.put("23", "出版");
        typeMap.put("24", "其他");
        List<Book> bookList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JSONObject bookObject = list.getJSONObject(i);
            String author_name = bookObject.getString("author_name");
            String book_brief = bookObject.getString("book_brief");
            String book_id = bookObject.getString("book_id");
            String book_name = bookObject.getString("book_name");
            String chapter = bookObject.getString("chapter");
            String charge = bookObject.getString("charge");
            String cover = bookObject.getString("cover");
            String ctype_id = bookObject.getString("ctype_id");
            String gender = bookObject.getString("gender");
            String is_show = bookObject.getString("is_show");
            String state = bookObject.getString("state");
            String type_id = bookObject.getString("type_id");
            String words = bookObject.getString("words");
            //章节列表
            String chaptersStr = HttpTool.sendGet("http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + book_id, "");
            JSONArray listChapters = JSON.parseArray(chaptersStr);
            JSONObject firstChapters = listChapters.getJSONObject(1);
            String first_chapter_id = firstChapters.getString("content_id");
            String first_chapter_name = firstChapters.getString("name");
            JSONObject lastChapters = listChapters.getJSONObject(listChapters.size() - 1);
            String last_chapter_id = lastChapters.getString("content_id");
            String last_chapter_name = lastChapters.getString("name");

            Book book = new Book();
            book.setId(Long.parseLong(book_id));
            book.setBookCategoryId(Integer.parseInt(type_id));
            book.setName(book_name);
            book.setRealname(book_name);
            book.setAuthor(author_name);
            book.setImage(cover);
            book.setDescription(book_brief);
            book.setSex(gender);
            book.setBillingType(String.valueOf(Integer.parseInt(charge) + 1));
            book.setWordCount(words);
            book.setChapterNum(Integer.valueOf(chapter));
//            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis()+""));
            book.setCreatetime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setCpId(Integer.parseInt(book_id));
            book.setCpName(book_name);
            book.setTags(typeMap.get(ctype_id));
            book.setState("1");
            book.setPrice(45);
            book.setArticleChapterOrder(4);
            book.setAttentChapterOrder(4);
            book.setCornerMark("hot");
            book.setIsFinish(String.valueOf(Integer.parseInt(state) - 1));
            book.setReferralNum(new Byte((byte) 5));
            book.setPutAdSet(new Byte((byte) 1));
            book.setClassifyWhiteList(new Byte((byte) 0));
            book.setCheckRank(new Byte((byte) 0));
            book.setRank(new Byte((byte) 6));
            book.setExpireTime(Integer.MAX_VALUE);
            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setAppPrice(45);
            book.setFirstChapterId(Long.parseLong(first_chapter_id));
            book.setFirstChapterName(first_chapter_name);
            book.setLastChapterId(Long.parseLong(last_chapter_id));
            book.setLastChapterName(last_chapter_name);
            book.setCansee("1");
            book.setRank(new Byte((byte) 6));

            bookList.add(book);


        }

        for (Book book : bookList) {
            int i = new Random().nextInt(bookList.size());
            book.setRecommandBookId(bookList.get(i).getId());
            System.out.println(JSON.toJSONString(book));
            bookMapper.insert(book);


        }

    }

    @Override
    public List<Chapter> getChaptersByBookId(String bookId) {
        List<Chapter> list = new ArrayList<>();
        try {
            if (StringUtils.isEmpty(bookId)){
                throw new IllegalArgumentException("bookId cannot be null or empty");
            }
            final   String chaptersUrl = "http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + bookId;
            String result = HttpTool.sendGet(chaptersUrl, "");
            if (StringUtils.isEmpty(result)) {
                return list;
            }
            JSONArray jsonArray = JSON.parseArray(result);
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                Chapter chapter = new Chapter();
                chapter.setBookId(jsonObject.getString("book_id"));
                chapter.setContentId(jsonObject.getString("content_id"));
                chapter.setIsPay(jsonObject.getString("ispay"));
                chapter.setName(jsonObject.getString("name"));
                chapter.setNum(jsonObject.getString("num"));
                chapter.setType(jsonObject.getString("type"));
                chapter.setVolumeId(jsonObject.getString("volume_id"));
                chapter.setWords(jsonObject.getString("words"));
                list.add(chapter);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            e.printStackTrace();

            return list;
        }

        return list;
    }


}
